@@ -157,20 +157,14 @@ before_first:
 		cursor->rel_pos = BTR_PCUR_ON;
 	}
 
-	if (index->is_ibuf()) {
-		ut_ad(!index->table->not_redundant());
-		cursor->old_n_fields = uint16_t(rec_get_n_fields_old(rec));
-	} else {
-		cursor->old_n_fields = static_cast<uint16>(
-			dict_index_get_n_unique_in_tree(index));
-		if (index->is_spatial() && !page_rec_is_leaf(rec)) {
-			ut_ad(dict_index_get_n_unique_in_tree_nonleaf(index)
-			      == DICT_INDEX_SPATIAL_NODEPTR_SIZE);
-			/* For R-tree, we have to compare
-			the child page numbers as well. */
-			cursor->old_n_fields
-				= DICT_INDEX_SPATIAL_NODEPTR_SIZE + 1;
-		}
+	cursor->old_n_fields = static_cast<uint16>(
+		dict_index_get_n_unique_in_tree(index));
+	if (index->is_spatial() && !page_rec_is_leaf(rec)) {
+		ut_ad(dict_index_get_n_unique_in_tree_nonleaf(index)
+		      == DICT_INDEX_SPATIAL_NODEPTR_SIZE);
+		/* For R-tree, we have to compare
+		the child page numbers as well. */
+		cursor->old_n_fields = DICT_INDEX_SPATIAL_NODEPTR_SIZE + 1;
 	}
 
 	cursor->old_n_core_fields = index->n_core_fields;
@@ -542,7 +536,7 @@ btr_pcur_move_to_next_page(
 	buf_block_t* next_block = btr_block_get(
 		*cursor->index(), next_page_no,
 		rw_lock_type_t(cursor->latch_mode & (RW_X_LATCH | RW_S_LATCH)),
-		page_is_leaf(page), mtr, &err);
+		mtr, &err);
 
 	if (UNIV_UNLIKELY(!next_block)) {
 		return err;
