@@ -385,14 +385,15 @@ we put it to free list to be used.
   * scan whole LRU list
   * scan LRU list even if buf_pool.try_LRU_scan is not set
 
-@param have_mutex  whether buf_pool.mutex is already being held
-@return the free control block, in state BUF_BLOCK_MEMORY */
-buf_block_t *buf_LRU_get_free_block(bool have_mutex)
+@param get  how to allocate the block
+@return the free control block, in state BUF_BLOCK_MEMORY
+@retval nullptr if get==have_no_mutex_soft and memory was not available */
+buf_block_t* buf_LRU_get_free_block(buf_LRU_get get)
 {
 	ulint		n_iterations	= 0;
 	ulint		flush_failures	= 0;
 	MONITOR_INC(MONITOR_LRU_GET_FREE_SEARCH);
-	if (have_mutex) {
+	if (UNIV_UNLIKELY(get == have_mutex)) {
 		mysql_mutex_assert_owner(&buf_pool.mutex);
 		goto got_mutex;
 	}
@@ -411,13 +412,14 @@ got_mutex:
 	DBUG_EXECUTE_IF("ib_lru_force_no_free_page",
 		if (!buf_lru_free_blocks_error_printed) {
 			n_iterations = 21;
+			block = nullptr;
 			goto not_found;});
 
 retry:
 	/* If there is a block in the free list, take it */
 	if ((block = buf_LRU_get_free_only()) != nullptr) {
 got_block:
-		if (!have_mutex) {
+		if (UNIV_LIKELY(get != have_mutex)) {
 			mysql_mutex_unlock(&buf_pool.mutex);
 		}
 		block->page.zip.clear();
@@ -441,6 +443,11 @@ got_block:
 		buf_pool.try_LRU_scan = false;
 	}
 
+	if (get == have_no_mutex_soft) {
+		mysql_mutex_unlock(&buf_pool.mutex);
+		return nullptr;
+	}
+
 	for (;;) {
 		if ((block = buf_LRU_get_free_only()) != nullptr) {
 			goto got_block;
@@ -853,7 +860,6 @@ relocate_compressed:
 			ut_d(uint32_t s=) b->fix();
 			ut_ad(s == buf_page_t::FREED
 			      || s == buf_page_t::UNFIXED
-			      || s == buf_page_t::IBUF_EXIST
 			      || s == buf_page_t::REINIT);
 		}
 		break;
@@ -1324,6 +1330,51 @@ bool buf_LRU_scan_and_free_block(ulint limit)
     buf_LRU_free_from_common_LRU_list(limit);
 }
 
+void buf_LRU_truncate_temp(uint32_t threshold)
+{
+  /* Set the extent descriptor page state as FREED */
+  for (uint32_t cur_xdes_page= xdes_calc_descriptor_page(
+         0, fil_system.temp_space->free_limit);
+       cur_xdes_page >= threshold;)
+  {
+    mtr_t mtr;
+    mtr.start();
+    if (buf_block_t* block= buf_page_get_gen(
+          page_id_t(SRV_TMP_SPACE_ID, cur_xdes_page), 0, RW_X_LATCH,
+          nullptr, BUF_PEEK_IF_IN_POOL, &mtr))
+    {
+      uint32_t state= block->page.state();
+      ut_ad(state > buf_page_t::UNFIXED);
+      ut_ad(state < buf_page_t::READ_FIX);
+      block->page.set_freed(state);
+    }
+    cur_xdes_page-= uint32_t(srv_page_size);
+    mtr.commit();
+  }
+
+  const page_id_t limit{SRV_TMP_SPACE_ID, threshold};
+  mysql_mutex_lock(&buf_pool.mutex);
+  for (buf_page_t* bpage = UT_LIST_GET_FIRST(buf_pool.LRU);
+       bpage;)
+  {
+    buf_page_t* next= UT_LIST_GET_NEXT(LRU, bpage);
+    if (bpage->id() >= limit)
+    {
+    #ifdef UNIV_DEBUG
+      if (bpage->lock.u_lock_try(0))
+      {
+        ut_ad(bpage->state() == buf_page_t::FREED);
+        bpage->lock.u_unlock();
+      }
+    #endif /* UNIV_DEBUG */
+      ut_ad(!reinterpret_cast<buf_block_t*>(bpage)->index);
+      buf_LRU_free_page(bpage, true);
+    }
+    bpage= next;
+  }
+  mysql_mutex_unlock(&buf_pool.mutex);
+}
+
 #ifdef UNIV_DEBUG
 /** Validate the LRU list. */
 void buf_LRU_validate()
