@@ -33,7 +33,6 @@ Refactored 2013-7-26 by Kevin Lewis
 #include "os0file.h"
 #include "row0mysql.h"
 #include "buf0dblwr.h"
-#include "log.h"
 
 /** The server header file is included to access opt_initialize global variable.
 If server passes the option for create/open DB to SE, we should remove such
@@ -101,6 +100,7 @@ SysTablespace::parse_params(
 
 	ut_ad(m_last_file_size_max == 0);
 	ut_ad(!m_auto_extend_last_file);
+	ut_ad(!m_auto_shrink);
 
 	char*	new_str = mem_strdup(filepath_spec);
 	char*	str = new_str;
@@ -147,6 +147,11 @@ SysTablespace::parse_params(
 				str = parse_units(str, &size);
 			}
 
+		        if (0 == strncmp(str, ":autoshrink",
+	                         (sizeof ":autoshrink") - 1)) {
+			   str += (sizeof ":autoshrink") - 1;
+			}
+
 			if (*str != '\0') {
 				ut_free(new_str);
 				ib::error()
@@ -267,6 +272,12 @@ SysTablespace::parse_params(
 				str = parse_units(str, &m_last_file_size_max);
 			}
 
+		        if (0 == strncmp(str, ":autoshrink",
+	                         (sizeof ":autoshrink") - 1)) {
+			   str += (sizeof ":autoshrink") - 1;
+			   m_auto_shrink = true;
+			}
+
 			if (*str != '\0') {
 				ut_free(new_str);
 				ib::error() << "syntax error in file path or"
@@ -334,6 +345,7 @@ SysTablespace::shutdown()
 	m_created_new_raw = 0;
 	m_is_tablespace_full = false;
 	m_sanity_checks_done = false;
+	m_auto_shrink = false;
 }
 
 /** Verify the size of the physical file.
@@ -569,7 +581,7 @@ inline dberr_t SysTablespace::read_lsn_and_check_flags()
 	}
 
 	err = it->read_first_page(
-		m_ignore_read_only && srv_read_only_mode);
+		m_ignore_read_only ?  false : srv_read_only_mode);
 
 	if (err != DB_SUCCESS) {
 		return(err);
@@ -583,62 +595,47 @@ inline dberr_t SysTablespace::read_lsn_and_check_flags()
 
 	/* Check the contents of the first page of the
 	first datafile. */
-	err = it->validate_first_page();
+	for (int retry = 0; retry < 2; ++retry) {
 
-	if (err != DB_SUCCESS) {
-		if (recv_sys.dblwr.restore_first_page(
+		err = it->validate_first_page();
+
+		if (err != DB_SUCCESS
+		    && (retry == 1
+			|| recv_sys.dblwr.restore_first_page(
 				it->m_space_id, it->m_filepath,
-				it->handle())) {
+				it->handle()))) {
+
 			it->close();
+
 			return(err);
 		}
-		err = it->read_first_page(
-			m_ignore_read_only && srv_read_only_mode);
 	}
 
 	/* Make sure the tablespace space ID matches the
 	space ID on the first page of the first datafile. */
-	if (err != DB_SUCCESS || space_id() != it->m_space_id) {
-		sql_print_error("InnoDB: The data file '%s'"
-				" has the wrong space ID."
-				" It should be " UINT32PF ", but " UINT32PF
-				" was found", it->filepath(),
-				space_id(), it->m_space_id);
+	if (space_id() != it->m_space_id) {
+
+		ib::error()
+			<< "The data file '" << it->filepath()
+			<< "' has the wrong space ID. It should be "
+			<< space_id() << ", but " << it->m_space_id
+			<< " was found";
+
 		it->close();
-		return err;
+
+		return(err);
 	}
 
-	if (srv_force_recovery != 6
-	    && srv_operation == SRV_OPERATION_NORMAL
-	    && !log_sys.next_checkpoint_lsn
-	    && log_sys.format == log_t::FORMAT_3_23) {
-
-		log_sys.latch.wr_lock(SRW_LOCK_CALL);
+	if (srv_operation == SRV_OPERATION_NORMAL) {
 		/* Prepare for possible upgrade from 0-sized ib_logfile0. */
+		ut_ad(!log_sys.next_checkpoint_lsn);
 		log_sys.next_checkpoint_lsn = mach_read_from_8(
 			it->m_first_page + 26/*FIL_PAGE_FILE_FLUSH_LSN*/);
-		if (log_sys.next_checkpoint_lsn < 8204) {
-			/* Before MDEV-14425, InnoDB had a minimum LSN
-			of 8192+12=8204. Likewise, mariadb-backup
-			--prepare would create an empty ib_logfile0
-			after applying the log. We will allow an
-			upgrade from such an empty log. */
-			sql_print_error("InnoDB: ib_logfile0 is "
-					"empty, and LSN is unknown.");
-			err = DB_CORRUPTION;
-		} else {
-			log_sys.last_checkpoint_lsn =
-				recv_sys.lsn = recv_sys.file_checkpoint =
-				log_sys.next_checkpoint_lsn;
-			log_sys.set_recovered_lsn(log_sys.next_checkpoint_lsn);
-			log_sys.next_checkpoint_no = 0;
-		}
-
-		log_sys.latch.wr_unlock();
 	}
 
 	it->close();
-	return err;
+
+	return(DB_SUCCESS);
 }
 
 /** Check if a file can be opened in the correct mode.
@@ -992,6 +989,7 @@ SysTablespace::normalize_size()
 	for (files_t::iterator it = m_files.begin(); it != end; ++it) {
 
 		it->m_size <<= (20U - srv_page_size_shift);
+		it->m_user_param_size = it->m_size;
 	}
 
 	m_last_file_size_max <<= (20U - srv_page_size_shift);
