@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2013, 2022, MariaDB Corporation.
+Copyright (c) 2013, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -51,35 +51,6 @@ using space_list_t= ilist<fil_space_t, space_list_tag_t>;
 // Forward declaration
 extern my_bool srv_use_doublewrite_buf;
 
-/** Possible values of innodb_flush_method */
-enum srv_flush_t
-{
-  /** fsync, the default */
-  SRV_FSYNC= 0,
-  /** open log files in O_DSYNC mode */
-  SRV_O_DSYNC,
-  /** do not call os_file_flush() when writing data files, but do flush
-  after writing to log files */
-  SRV_LITTLESYNC,
-  /** do not flush after writing */
-  SRV_NOSYNC,
-  /** invoke os_file_set_nocache() on data files. This implies using
-  unbuffered I/O but still fdatasync(), because some filesystems might
-  not flush meta-data on write completion */
-  SRV_O_DIRECT,
-  /** Like O_DIRECT, but skip fdatasync(), assuming that the data is
-  durable on write completion */
-  SRV_O_DIRECT_NO_FSYNC
-#ifdef _WIN32
-  /** Traditional Windows appoach to open all files without caching,
-  and do FileFlushBuffers() */
-  ,SRV_ALL_O_DIRECT_FSYNC
-#endif
-};
-
-/** innodb_flush_method */
-extern ulong srv_file_flush_method;
-
 /** Undo tablespaces starts with space_id. */
 extern uint32_t srv_undo_space_id_start;
 /** The number of UNDO tablespaces that are open and ready to use. */
@@ -394,16 +365,22 @@ private:
   /** Number of pending operations on the file.
   The tablespace cannot be freed while (n_pending & PENDING) != 0. */
   std::atomic<uint32_t> n_pending;
+  /** Flag in n_pending that indicates that the tablespace is about to be
+  deleted, and no further operations should be performed */
+  static constexpr uint32_t STOPPING_READS= 1U << 31;
   /** Flag in n_pending that indicates that the tablespace is being
   deleted, and no further operations should be performed */
-  static constexpr uint32_t STOPPING= 1U << 31;
+  static constexpr uint32_t STOPPING_WRITES= 1U << 30;
+  /** Flags in n_pending that indicate that the tablespace is being
+  deleted, and no further operations should be performed */
+  static constexpr uint32_t STOPPING= STOPPING_READS | STOPPING_WRITES;
   /** Flag in n_pending that indicates that the tablespace is a candidate
   for being closed, and fil_node_t::is_open() can only be trusted after
   acquiring fil_system.mutex and resetting the flag */
-  static constexpr uint32_t CLOSING= 1U << 30;
+  static constexpr uint32_t CLOSING= 1U << 29;
   /** Flag in n_pending that indicates that the tablespace needs fsync().
   This must be the least significant flag bit; @see release_flush() */
-  static constexpr uint32_t NEEDS_FSYNC= 1U << 29;
+  static constexpr uint32_t NEEDS_FSYNC= 1U << 28;
   /** The reference count */
   static constexpr uint32_t PENDING= ~(STOPPING | CLOSING | NEEDS_FSYNC);
   /** latch protecting all page allocation bitmap pages */
@@ -515,20 +492,19 @@ public:
   /** Close each file. Only invoked on fil_system.temp_space. */
   void close();
 
-  /** Note that operations on the tablespace must stop.
-  @return whether the operations were already stopped */
-  inline bool set_stopping_check();
   /** Note that operations on the tablespace must stop. */
   inline void set_stopping();
 
   /** Note that operations on the tablespace can resume after truncation */
   inline void clear_stopping();
 
-  /** Look up the tablespace and wait for pending operations to cease
-  @param id  tablespace identifier
-  @return tablespace
-  @retval nullptr if no tablespace was found */
-  static fil_space_t *check_pending_operations(uint32_t id);
+  /** Drop the tablespace and wait for any pending operations to cease
+  @param id               tablespace identifier
+  @param detached_handle  pointer to file to be closed later, or nullptr
+  @return tablespace to invoke fil_space_free() on
+  @retval nullptr if no tablespace was found, or it was deleted by
+  another concurrent thread */
+  static fil_space_t *drop(uint32_t id, pfs_os_file_t *detached_handle);
 
 private:
   MY_ATTRIBUTE((warn_unused_result))
@@ -552,13 +528,19 @@ public:
 
   MY_ATTRIBUTE((warn_unused_result))
   /** Acquire a tablespace reference for I/O.
+  @param avoid   when these flags are set, nothing will be acquired
   @return whether the file is usable */
-  bool acquire()
+  bool acquire(uint32_t avoid= STOPPING | CLOSING)
   {
-    const auto flags= acquire_low(STOPPING | CLOSING) & (STOPPING | CLOSING);
+    const auto flags= acquire_low(avoid) & (avoid);
     return UNIV_LIKELY(!flags) || (flags == CLOSING && acquire_and_prepare());
   }
 
+  /** Acquire a tablespace reference for writing.
+  @param avoid   when these flags are set, nothing will be acquired
+  @return whether the file is writable */
+  bool acquire_for_write() { return acquire(STOPPING_WRITES | CLOSING); }
+
   /** Acquire another tablespace reference for I/O. */
   inline void reacquire();
 
@@ -575,12 +557,12 @@ public:
   void clear_flush()
   {
 #if defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-    static_assert(NEEDS_FSYNC == 1U << 29, "compatibility");
-    __asm__ __volatile__("lock btrl $29, %0" : "+m" (n_pending));
+    static_assert(NEEDS_FSYNC == 1U << 28, "compatibility");
+    __asm__ __volatile__("lock btrl $28, %0" : "+m" (n_pending));
 #elif defined _MSC_VER && (defined _M_IX86 || defined _M_X64)
-    static_assert(NEEDS_FSYNC == 1U << 29, "compatibility");
+    static_assert(NEEDS_FSYNC == 1U << 28, "compatibility");
     _interlockedbittestandreset(reinterpret_cast<volatile long*>
-                                (&n_pending), 29);
+                                (&n_pending), 28);
 #else
     n_pending.fetch_and(~NEEDS_FSYNC, std::memory_order_release);
 #endif
@@ -591,12 +573,12 @@ private:
   void clear_closing()
   {
 #if defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-    static_assert(CLOSING == 1U << 30, "compatibility");
-    __asm__ __volatile__("lock btrl $30, %0" : "+m" (n_pending));
+    static_assert(CLOSING == 1U << 29, "compatibility");
+    __asm__ __volatile__("lock btrl $29, %0" : "+m" (n_pending));
 #elif defined _MSC_VER && (defined _M_IX86 || defined _M_X64)
-    static_assert(CLOSING == 1U << 30, "compatibility");
+    static_assert(CLOSING == 1U << 29, "compatibility");
     _interlockedbittestandreset(reinterpret_cast<volatile long*>
-                                (&n_pending), 30);
+                                (&n_pending), 29);
 #else
     n_pending.fetch_and(~CLOSING, std::memory_order_relaxed);
 #endif
@@ -607,8 +589,10 @@ private:
 public:
   /** @return whether close() of the file handle has been requested */
   bool is_closing() const { return pending() & CLOSING; }
-  /** @return whether the tablespace is going to be dropped */
+  /** @return whether the tablespace is about to be dropped */
   bool is_stopping() const { return pending() & STOPPING; }
+  /** @return whether the tablespace is going to be dropped */
+  bool is_stopping_writes() const { return pending() & STOPPING_WRITES; }
   /** @return number of pending operations */
   bool is_ready_to_close() const
   { return (pending() & (PENDING | CLOSING)) == CLOSING; }
@@ -617,7 +601,7 @@ public:
   /** @return whether fsync() or similar is needed, and the tablespace is
   not being dropped  */
   bool needs_flush_not_stopping() const
-  { return (pending() & (NEEDS_FSYNC | STOPPING)) == NEEDS_FSYNC; }
+  { return (pending() & (NEEDS_FSYNC | STOPPING_WRITES)) == NEEDS_FSYNC; }
 
   uint32_t referenced() const { return pending() & PENDING; }
 private:
@@ -630,6 +614,8 @@ private:
   }
 
 public:
+  /** Reopen all files on set_write_through() or set_buffered(). */
+  static void reopen_all();
   /** Try to close a file to adhere to the innodb_open_files limit.
   @param print_info   whether to diagnose why a file cannot be closed
   @return whether a file was closed */
@@ -651,7 +637,7 @@ public:
                                               std::memory_order_relaxed))
     {
       ut_ad(n & PENDING);
-      if (n & (NEEDS_FSYNC | STOPPING))
+      if (n & (NEEDS_FSYNC | STOPPING_WRITES))
         return false;
     }
 
@@ -909,6 +895,11 @@ public:
   @return tablespace
   @retval nullptr if the tablespace is missing or inaccessible */
   static fil_space_t *get(uint32_t id);
+  /** Acquire a tablespace reference for writing.
+  @param id      tablespace identifier
+  @return tablespace
+  @retval nullptr if the tablespace is missing or inaccessible */
+  static fil_space_t *get_for_write(uint32_t id);
 
   /** Add/remove the free page in the freed ranges list.
   @param[in] offset     page number to be added
@@ -938,6 +929,11 @@ public:
     freed_ranges.add_range(range);
   }
 
+  /** Clear the freed range in temporary tablespace
+  which are in shrinking ranges.
+  @param threshold  to be truncated value*/
+  inline void clear_freed_ranges(uint32_t threshold);
+
   /** Set the tablespace size in pages */
   void set_sizes(uint32_t s)
   {
@@ -1273,11 +1269,11 @@ constexpr uint16_t FIL_PAGE_RTREE= 17854;
 constexpr uint16_t FIL_PAGE_UNDO_LOG= 2;
 /** Index node (of file-in-file metadata) */
 constexpr uint16_t FIL_PAGE_INODE= 3;
-/** Insert buffer free list */
+/** Former change buffer free list */
 constexpr uint16_t FIL_PAGE_IBUF_FREE_LIST= 4;
 /** Freshly allocated page */
 constexpr uint16_t FIL_PAGE_TYPE_ALLOCATED= 0;
-/** Change buffer bitmap (pages n*innodb_page_size+1) */
+/** Former change buffer bitmap pages (pages n*innodb_page_size+1) */
 constexpr uint16_t FIL_PAGE_IBUF_BITMAP= 5;
 /** System page */
 constexpr uint16_t FIL_PAGE_TYPE_SYS= 6;
@@ -1418,6 +1414,20 @@ public:
 	fil_space_t*	temp_space;	/*!< The innodb_temporary tablespace */
   /** Map of fil_space_t::id to fil_space_t* */
   hash_table_t spaces;
+
+  /** whether each write to data files is durable (O_DSYNC) */
+  my_bool write_through;
+  /** whether data files are buffered (not O_DIRECT) */
+  my_bool buffered;
+
+  /** Try to enable or disable write-through of data files */
+  void set_write_through(bool write_through);
+  /** Try to enable or disable file system caching of data files */
+  void set_buffered(bool buffered);
+
+  TPOOL_SUPPRESS_TSAN bool is_write_through() const { return write_through; }
+  TPOOL_SUPPRESS_TSAN bool is_buffered() const { return buffered; }
+
   /** tablespaces for which fil_space_t::needs_flush() holds */
   sized_ilist<fil_space_t, unflushed_spaces_tag_t> unflushed_spaces;
   /** number of currently open files; protected by mutex */
@@ -1522,52 +1532,27 @@ inline void fil_space_t::reacquire()
 #endif /* SAFE_MUTEX */
 }
 
-/** Note that operations on the tablespace must stop.
-@return whether the operations were already stopped */
-inline bool fil_space_t::set_stopping_check()
-{
-  mysql_mutex_assert_owner(&fil_system.mutex);
-#if (defined __clang_major__ && __clang_major__ < 10) || defined __APPLE_CC__
-  /* Only clang-10 introduced support for asm goto */
-  return n_pending.fetch_or(STOPPING, std::memory_order_relaxed) & STOPPING;
-#elif defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-  static_assert(STOPPING == 1U << 31, "compatibility");
-  __asm__ goto("lock btsl $31, %0\t\njnc %l1" : : "m" (n_pending)
-               : "cc", "memory" : not_stopped);
-  return true;
-not_stopped:
-  return false;
-#elif defined _MSC_VER && (defined _M_IX86 || defined _M_X64)
-  static_assert(STOPPING == 1U << 31, "compatibility");
-  return _interlockedbittestandset(reinterpret_cast<volatile long*>
-                                   (&n_pending), 31);
-#else
-  return n_pending.fetch_or(STOPPING, std::memory_order_relaxed) & STOPPING;
-#endif
-}
-
-/** Note that operations on the tablespace must stop.
-@return whether the operations were already stopped */
+/** Note that operations on the tablespace must stop. */
 inline void fil_space_t::set_stopping()
 {
   mysql_mutex_assert_owner(&fil_system.mutex);
 #if defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-  static_assert(STOPPING == 1U << 31, "compatibility");
-  __asm__ __volatile__("lock btsl $31, %0" : "+m" (n_pending));
+  static_assert(STOPPING_WRITES == 1U << 30, "compatibility");
+  __asm__ __volatile__("lock btsl $30, %0" : "+m" (n_pending));
 #elif defined _MSC_VER && (defined _M_IX86 || defined _M_X64)
-  static_assert(STOPPING == 1U << 31, "compatibility");
-  _interlockedbittestandset(reinterpret_cast<volatile long*>(&n_pending), 31);
+  static_assert(STOPPING_WRITES == 1U << 30, "compatibility");
+  _interlockedbittestandset(reinterpret_cast<volatile long*>(&n_pending), 30);
 #else
-  n_pending.fetch_or(STOPPING, std::memory_order_relaxed);
+  n_pending.fetch_or(STOPPING_WRITES, std::memory_order_relaxed);
 #endif
 }
 
 inline void fil_space_t::clear_stopping()
 {
   mysql_mutex_assert_owner(&fil_system.mutex);
-  static_assert(STOPPING == 1U << 31, "compatibility");
-  ut_d(auto n=) n_pending.fetch_sub(STOPPING, std::memory_order_relaxed);
-  ut_ad(n & STOPPING);
+  static_assert(STOPPING_WRITES == 1U << 30, "compatibility");
+  ut_d(auto n=) n_pending.fetch_sub(STOPPING_WRITES, std::memory_order_relaxed);
+  ut_ad((n & STOPPING) == STOPPING_WRITES);
 }
 
 /** Flush pending writes from the file system cache to the file. */
@@ -1576,12 +1561,7 @@ template<bool have_reference> inline void fil_space_t::flush()
   mysql_mutex_assert_not_owner(&fil_system.mutex);
   ut_ad(!have_reference || (pending() & PENDING));
   ut_ad(purpose == FIL_TYPE_TABLESPACE || purpose == FIL_TYPE_IMPORT);
-  if (srv_file_flush_method == SRV_O_DIRECT_NO_FSYNC)
-  {
-    ut_ad(!is_in_unflushed_spaces);
-    ut_ad(!needs_flush());
-  }
-  else if (have_reference)
+  if (have_reference)
     flush_low();
   else
   {
@@ -1821,4 +1801,10 @@ void test_make_filepath();
 @return	block size */
 ulint fil_space_get_block_size(const fil_space_t* space, unsigned offset);
 
+/** Check whether encryption key found
+@param crypt_data Encryption data
+@param f_name     File name
+@return encryption key found */
+bool fil_crypt_check(fil_space_crypt_t *crypt_data, const char *f_name);
+
 #endif /* UNIV_INNOCHECKSUM */
