@@ -71,136 +71,77 @@ int spider_udf_set_copy_tables_param_default(
   DBUG_RETURN(0);
 }
 
-#define SPIDER_PARAM_STR_LEN(name) name ## _length
-#define SPIDER_PARAM_STR(title_name, param_name) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
-  { \
-    DBUG_PRINT("info",("spider " title_name " start")); \
-    if (!copy_tables->param_name) \
-    { \
-      if ((copy_tables->param_name = spider_get_string_between_quote( \
-        start_ptr, TRUE, &param_string_parse))) \
-        copy_tables->SPIDER_PARAM_STR_LEN(param_name) = \
-          strlen(copy_tables->param_name); \
-      else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+#define SPIDER_PARAM_LEN(name) name ## _length
+#define SPIDER_PARAM_STR(title_name, param_name)                        \
+  if (!strncasecmp(parse.start_title, title_name, title_length))        \
+  {                                                                     \
+    DBUG_PRINT("info",("spider " title_name " start"));                 \
+    if (!copy_tables->param_name)                                       \
+    {                                                                   \
+      if ((copy_tables->param_name = spider_create_string(parse.start_value, \
+                                                          value_length))) \
+        copy_tables->SPIDER_PARAM_LEN(param_name) = strlen(copy_tables->param_name); \
+      else {                                                            \
+        error_num= parse.fail(true);                                    \
+        goto error;                                                     \
+      }                                                                 \
       DBUG_PRINT("info",("spider " title_name "=%s", copy_tables->param_name)); \
     } \
     break; \
   }
-#define SPIDER_PARAM_HINT_WITH_MAX(title_name, param_name, check_length, max_size, min_val, max_val) \
-  if (!strncasecmp(tmp_ptr, title_name, check_length)) \
-  { \
-    DBUG_PRINT("info",("spider " title_name " start")); \
-    DBUG_PRINT("info",("spider max_size=%d", max_size)); \
-    int hint_num = atoi(tmp_ptr + check_length) - 1; \
-    DBUG_PRINT("info",("spider hint_num=%d", hint_num)); \
-    DBUG_PRINT("info",("spider copy_tables->param_name=%x", \
-      copy_tables->param_name)); \
-    if (copy_tables->param_name) \
-    { \
-      if (hint_num < 0 || hint_num >= max_size) \
-      { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } else if (copy_tables->param_name[hint_num] != -1) \
-        break; \
-      char *hint_str = spider_get_string_between_quote(start_ptr, FALSE); \
-      if (hint_str) \
-      { \
-        copy_tables->param_name[hint_num] = atoi(hint_str); \
-        if (copy_tables->param_name[hint_num] < min_val) \
-          copy_tables->param_name[hint_num] = min_val; \
-        else if (copy_tables->param_name[hint_num] > max_val) \
-          copy_tables->param_name[hint_num] = max_val; \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
-      DBUG_PRINT("info",("spider " title_name "[%d]=%d", hint_num, \
-        copy_tables->param_name[hint_num])); \
-    } else { \
-      error_num = param_string_parse.print_param_error(); \
-      goto error; \
-    } \
-    break; \
-  }
 #define SPIDER_PARAM_INT_WITH_MAX(title_name, param_name, min_val, max_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (copy_tables->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        copy_tables->param_name = atoi(tmp_ptr2); \
-        if (copy_tables->param_name < min_val) \
-          copy_tables->param_name = min_val; \
-        else if (copy_tables->param_name > max_val) \
-          copy_tables->param_name = max_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      copy_tables->param_name = atoi(parse.start_value); \
+      if (copy_tables->param_name < min_val) \
+        copy_tables->param_name = min_val; \
+      else if (copy_tables->param_name > max_val) \
+        copy_tables->param_name = max_val; \
       DBUG_PRINT("info",("spider " title_name "=%d", copy_tables->param_name)); \
     } \
     break; \
   }
 #define SPIDER_PARAM_INT(title_name, param_name, min_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (copy_tables->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        copy_tables->param_name = atoi(tmp_ptr2); \
-        if (copy_tables->param_name < min_val) \
-          copy_tables->param_name = min_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      copy_tables->param_name = atoi(parse.start_value); \
+      if (copy_tables->param_name < min_val) \
+        copy_tables->param_name = min_val; \
       DBUG_PRINT("info",("spider " title_name "=%d", copy_tables->param_name)); \
     } \
     break; \
   }
 #define SPIDER_PARAM_LONGLONG(title_name, param_name, min_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (copy_tables->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        copy_tables->param_name = \
-          my_strtoll10(tmp_ptr2, (char**) NULL, &error_num); \
-        if (copy_tables->param_name < min_val) \
-          copy_tables->param_name = min_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      copy_tables->param_name = \
+        my_strtoll10(parse.start_value, (char**) NULL, &error_num); \
+      if (copy_tables->param_name < min_val) \
+        copy_tables->param_name = min_val; \
       DBUG_PRINT("info",("spider " title_name "=%lld", \
-        copy_tables->param_name)); \
+                         copy_tables->param_name)); \
     } \
     break; \
   }
 
+static void spider_minus_1(SPIDER_COPY_TABLES *copy_tables)
+{
+  copy_tables->bulk_insert_interval = -1;
+  copy_tables->bulk_insert_rows = -1;
+  copy_tables->use_table_charset = -1;
+  copy_tables->use_transaction = -1;
+  copy_tables->bg_mode = -1;
+}
+
 int spider_udf_parse_copy_tables_param(
   SPIDER_COPY_TABLES *copy_tables,
   char *param,
@@ -208,67 +149,40 @@ int spider_udf_parse_copy_tables_param(
 ) {
   int error_num = 0;
   char *param_string = NULL;
-  char *sprit_ptr;
-  char *tmp_ptr, *tmp_ptr2, *start_ptr;
-  int title_length;
-  SPIDER_PARAM_STRING_PARSE param_string_parse;
+  char *start_param;
+  int title_length, value_length;
+  SPIDER_PARAM_STRING_PARSE parse;
   DBUG_ENTER("spider_udf_parse_copy_tables_param");
-  copy_tables->bulk_insert_interval = -1;
-  copy_tables->bulk_insert_rows = -1;
-  copy_tables->use_table_charset = -1;
-  copy_tables->use_transaction = -1;
-  copy_tables->bg_mode = -1;
-
+  spider_minus_1(copy_tables);
   if (param_length == 0)
     goto set_default;
   DBUG_PRINT("info",("spider create param_string string"));
-  if (
-    !(param_string = spider_create_string(
-      param,
-      param_length))
-  ) {
+  if (!(param_string = spider_create_string(param, param_length)))
+  {
     error_num = HA_ERR_OUT_OF_MEM;
     my_error(ER_OUT_OF_RESOURCES, MYF(0), HA_ERR_OUT_OF_MEM);
     goto error_alloc_param_string;
   }
   DBUG_PRINT("info",("spider param_string=%s", param_string));
 
-  sprit_ptr = param_string;
-  param_string_parse.init(param_string, ER_SPIDER_INVALID_UDF_PARAM_NUM);
-  while (sprit_ptr)
+  start_param = param_string;
+  parse.error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
+  while (*start_param != '\0')
   {
-    tmp_ptr = sprit_ptr;
-    while (*tmp_ptr == ' ' || *tmp_ptr == '\r' ||
-      *tmp_ptr == '\n' || *tmp_ptr == '\t')
-      tmp_ptr++;
-
-    if (*tmp_ptr == '\0')
-      break;
-
-    title_length = 0;
-    start_ptr = tmp_ptr;
-    while (*start_ptr != ' ' && *start_ptr != '\'' &&
-      *start_ptr != '"' && *start_ptr != '\0' &&
-      *start_ptr != '\r' && *start_ptr != '\n' &&
-      *start_ptr != '\t')
-    {
-      title_length++;
-      start_ptr++;
-    }
-    param_string_parse.set_param_title(tmp_ptr, tmp_ptr + title_length);
-    if ((error_num = param_string_parse.get_next_parameter_head(
-      start_ptr, &sprit_ptr)))
+    if (parse.locate_param_def(start_param))
     {
+      error_num= parse.fail(false);
       goto error;
     }
+    /* Null the end of the parameter value. */
+    *parse.end_value= '\0';
+    value_length= (int) (parse.end_value - parse.start_value);
 
-    switch (title_length)
+    switch (title_length = (int) (parse.end_title - parse.start_title))
     {
       case 0:
-        error_num = param_string_parse.print_param_error();
-        if (error_num)
-          goto error;
-        continue;
+        error_num= parse.fail(true);
+        goto error;
       case 3:
         SPIDER_PARAM_INT_WITH_MAX("bgm", bg_mode, 0, 1);
         SPIDER_PARAM_INT("bii", bulk_insert_interval, 0);
@@ -276,53 +190,43 @@ int spider_udf_parse_copy_tables_param(
         SPIDER_PARAM_STR("dtb", database);
         SPIDER_PARAM_INT_WITH_MAX("utc", use_table_charset, 0, 1);
         SPIDER_PARAM_INT_WITH_MAX("utr", use_transaction, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 7:
         SPIDER_PARAM_INT_WITH_MAX("bg_mode", bg_mode, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 8:
         SPIDER_PARAM_STR("database", database);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 15:
         SPIDER_PARAM_INT_WITH_MAX("use_transaction", use_transaction, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 16:
         SPIDER_PARAM_LONGLONG("bulk_insert_rows", bulk_insert_rows, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 17:
         SPIDER_PARAM_INT_WITH_MAX(
           "use_table_charset", use_table_charset, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 20:
         SPIDER_PARAM_INT("bulk_insert_interval", bulk_insert_interval, 0);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       default:
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
     }
-
-    /* Verify that the remainder of the parameter value is whitespace */
-    if ((error_num = param_string_parse.has_extra_parameter_values()))
-      goto error;
+    /* Restore delim */
+    *parse.end_value= parse.delim_value;
   }
 
 set_default:
-  if ((error_num = spider_udf_set_copy_tables_param_default(
-    copy_tables
-  )))
-    goto error;
-
-  if (param_string)
-    spider_free(spider_current_trx, param_string, MYF(0));
-  DBUG_RETURN(0);
-
+  error_num = spider_udf_set_copy_tables_param_default(copy_tables);
 error:
   if (param_string)
     spider_free(spider_current_trx, param_string, MYF(0));
@@ -353,7 +257,7 @@ int spider_udf_get_copy_tgt_tables(
     !(table_tables = spider_open_sys_table(
       thd, SPIDER_SYS_TABLES_TABLE_NAME_STR,
       SPIDER_SYS_TABLES_TABLE_NAME_LEN, FALSE, &open_tables_backup,
-      need_lock, &error_num))
+      &error_num))
   ) {
     my_error(error_num, MYF(0));
     goto error;
@@ -396,7 +300,7 @@ int spider_udf_get_copy_tgt_tables(
       (error_num = spider_get_sys_tables_connect_info(
         table_tables, tmp_share, 0, mem_root)) ||
       (error_num = spider_get_sys_tables_link_status(
-        table_tables, tmp_share, 0, mem_root)) ||
+        table_tables, tmp_share->link_statuses, mem_root)) ||
       (error_num = spider_get_sys_tables_link_idx(
         table_tables, &table_conn->link_idx, mem_root))
     ) {
@@ -541,8 +445,7 @@ int spider_udf_get_copy_tgt_tables(
     error_num = spider_sys_index_next_same(table_tables, table_key);
   } while (error_num == 0);
   spider_sys_index_end(table_tables);
-  spider_close_sys_table(thd, table_tables,
-    &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   table_tables = NULL;
 
   if (!copy_tables->table_conn[0])
@@ -564,8 +467,7 @@ int spider_udf_get_copy_tgt_tables(
 
 error:
   if (table_tables)
-    spider_close_sys_table(thd, table_tables,
-      &open_tables_backup, need_lock);
+    spider_sys_close_table(thd, &open_tables_backup);
   if (table_conn)
   {
     spider_free_tmp_dbton_share(tmp_share);
@@ -591,11 +493,10 @@ int spider_udf_get_copy_tgt_conns(
     while (table_conn)
     {
       share = table_conn->share;
-      if (
-        !(table_conn->conn = spider_get_conn(
-          share, 0, share->conn_keys[0], trx, NULL, FALSE, FALSE,
-          SPIDER_CONN_KIND_MYSQL, &error_num))
-      ) {
+      if (!(table_conn->conn=
+                spider_get_conn(share, 0, share->conn_keys[0], trx, NULL,
+                                FALSE, FALSE, &error_num)))
+      {
         my_error(ER_CONNECT_TO_FOREIGN_DATA_SOURCE, MYF(0), share->server_names[0]);
         DBUG_RETURN(ER_CONNECT_TO_FOREIGN_DATA_SOURCE);
       }
