@@ -85,7 +85,6 @@
 
 #define SPIDER_TEST(A) MY_TEST(A)
 
-#define SPIDER_FIELD_FIELDPTR_REQUIRES_THDPTR
 #define SPIDER_ENGINE_CONDITION_PUSHDOWN_IS_ALWAYS_ON
 
 #define SPIDER_Item_args_arg_count_IS_PROTECTED
@@ -108,7 +107,6 @@
 
 #define SPIDER_read_record_read_record(A) read_record()
 #define SPIDER_has_Item_with_subquery
-#define SPIDER_use_LEX_CSTRING_for_Field_blob_constructor
 #define SPIDER_use_LEX_CSTRING_for_database_tablename_alias
 #define SPIDER_THD_db_str(A) (A)->db.str
 #define SPIDER_THD_db_length(A) (A)->db.length
@@ -150,9 +148,14 @@ typedef start_new_trans *SPIDER_Open_tables_backup;
 #define spider_bit_is_set(BITMAP, BIT) \
   (uint) ((BITMAP)[(BIT) / 8] & (1 << ((BIT) & 7)))
 
+/* Change status of the remote backend server link. */
+/* 0 Doesn't change status.  */
 #define SPIDER_LINK_STATUS_NO_CHANGE         0
+/* 1 Changes status to OK.  */
 #define SPIDER_LINK_STATUS_OK                1
+/* 2 Changes status to RECOVERY.  */
 #define SPIDER_LINK_STATUS_RECOVERY          2
+/* 3 Changes status to no more in group communication. */
 #define SPIDER_LINK_STATUS_NG                3
 
 #define SPIDER_LINK_MON_OK                   0
@@ -168,279 +171,6 @@ typedef start_new_trans *SPIDER_Open_tables_backup;
 #define SPIDER_MEM_CALC_LIST_NUM           314
 #define SPIDER_CONN_META_BUF_LEN           64
 
-/*
-  IDs for spider mem alloc functions, including
-  - spider_alloc_calc_mem_init()
-  - spider_string::init_calc_mem()
-  - spider_malloc()
-  - spider_bulk_alloc_mem()
-  - spider_bulk_malloc()
-  In the format of
-  SPD_MID_<CALLSITE_FUNC_NAME_SANS_SPIDER_PREFIX>_<NO>
-*/
-enum spider_malloc_id {
-  SPD_MID_CHECK_HS_PK_UPDATE_1,
-  SPD_MID_COPY_TABLES_BODY_1,
-  SPD_MID_COPY_TABLES_BODY_2,
-  SPD_MID_COPY_TABLES_BODY_3,
-  SPD_MID_COPY_TABLES_BODY_4,
-  SPD_MID_COPY_TABLES_BODY_5,
-  SPD_MID_CREATE_CONN_1,
-  SPD_MID_CREATE_CONN_2,
-  SPD_MID_CREATE_CONN_3,
-  SPD_MID_CREATE_CONN_4,
-  SPD_MID_CREATE_CONN_5,
-  SPD_MID_CREATE_CONN_6,
-  SPD_MID_CREATE_CONN_KEYS_1,
-  SPD_MID_CREATE_CONN_THREAD_1,
-  SPD_MID_CREATE_LONGLONG_LIST_1,
-  SPD_MID_CREATE_LONG_LIST_1,
-  SPD_MID_CREATE_MON_THREADS_1,
-  SPD_MID_CREATE_MON_THREADS_2,
-  SPD_MID_CREATE_SHARE_1,
-  SPD_MID_CREATE_SHARE_2,
-  SPD_MID_CREATE_SPIDER_OBJECT_FOR_SHARE_1,
-  SPD_MID_CREATE_SPIDER_OBJECT_FOR_SHARE_2,
-  SPD_MID_CREATE_STRING_1,
-  SPD_MID_CREATE_STRING_LIST_1,
-  SPD_MID_CREATE_TABLE_HOLDER_1,
-  SPD_MID_CREATE_TABLE_NAME_STRING_1,
-  SPD_MID_CREATE_TRX_ALTER_TABLE_1,
-  SPD_MID_CREATE_TRX_HA_1,
-  SPD_MID_DB_CONN_QUEUE_ACTION_1,
-  SPD_MID_DB_FETCH_FOR_ITEM_SUM_FUNC_1,
-  SPD_MID_DB_FETCH_FOR_ITEM_SUM_FUNC_2,
-  SPD_MID_DB_FETCH_FOR_ITEM_SUM_FUNC_3,
-  SPD_MID_DB_HANDLERSOCKET_APPEND_REQUEST_KEY_1,
-  SPD_MID_DB_HANDLERSOCKET_EXEC_QUERY_1,
-  SPD_MID_DB_HANDLERSOCKET_INIT_1,
-  SPD_MID_DB_HANDLERSOCKET_RESULT_FETCH_ROW_FROM_TMP_TABLE_1,
-  SPD_MID_DB_HANDLERSOCKET_RESULT_FETCH_ROW_FROM_TMP_TABLE_2,
-  SPD_MID_DB_HANDLERSOCKET_ROW_APPEND_ESCAPED_TO_STR_1,
-  SPD_MID_DB_HANDLERSOCKET_ROW_CLONE_1,
-  SPD_MID_DB_HANDLERSOCKET_ROW_STORE_TO_FIELD_1,
-  SPD_MID_DB_HANDLERSOCKET_UTIL_APPEND_COLUMN_VALUE_1,
-  SPD_MID_DB_HANDLERSOCKET_UTIL_OPEN_ITEM_FUNC_1,
-  SPD_MID_DB_HANDLERSOCKET_UTIL_OPEN_ITEM_FUNC_2,
-  SPD_MID_DB_HANDLERSOCKET_UTIL_OPEN_ITEM_FUNC_3,
-  SPD_MID_DB_HS_STRING_REF_BUFFER_INIT_1,
-  SPD_MID_DB_HS_STR_BUFFER_ADD_1,
-  SPD_MID_DB_HS_STR_BUFFER_ADD_2,
-  SPD_MID_DB_HS_STR_BUFFER_INIT_1,
-  SPD_MID_DB_INIT_1,
-  SPD_MID_DB_INIT_10,
-  SPD_MID_DB_INIT_11,
-  SPD_MID_DB_INIT_12,
-  SPD_MID_DB_INIT_2,
-  SPD_MID_DB_INIT_3,
-  SPD_MID_DB_INIT_4,
-  SPD_MID_DB_INIT_5,
-  SPD_MID_DB_INIT_6,
-  SPD_MID_DB_INIT_7,
-  SPD_MID_DB_INIT_8,
-  SPD_MID_DB_INIT_9,
-  SPD_MID_DB_MARIADB_UTIL_APPEND_COLUMN_VALUE_1,
-  SPD_MID_DB_MARIADB_UTIL_APPEND_COLUMN_VALUE_2,
-  SPD_MID_DB_MBASE_EXEC_QUERY_1,
-  SPD_MID_DB_MBASE_EXEC_QUERY_2,
-  SPD_MID_DB_MBASE_INIT_1,
-  SPD_MID_DB_MBASE_INIT_2,
-  SPD_MID_DB_MBASE_RESULT_FETCH_ROW_FROM_TMP_TABLE_1,
-  SPD_MID_DB_MBASE_RESULT_FETCH_ROW_FROM_TMP_TABLE_2,
-  SPD_MID_DB_MBASE_RESULT_FETCH_ROW_FROM_TMP_TABLE_3,
-  SPD_MID_DB_MBASE_ROW_APPEND_ESCAPED_TO_STR_1,
-  SPD_MID_DB_MBASE_ROW_CLONE_1,
-  SPD_MID_DB_MBASE_SET_LOOP_CHECK_1,
-  SPD_MID_DB_MBASE_SET_SQL_MODE_1,
-  SPD_MID_DB_MBASE_SET_TIME_ZONE_1,
-  SPD_MID_DB_MBASE_SET_WAIT_TIMEOUT_1,
-  SPD_MID_DB_MBASE_UTIL_PRINT_ITEM_FUNC_1,
-  SPD_MID_DB_MBASE_UTIL_PRINT_ITEM_FUNC_2,
-  SPD_MID_DB_MBASE_UTIL_PRINT_ITEM_FUNC_3,
-  SPD_MID_DB_MBASE_XA_COMMIT_1,
-  SPD_MID_DB_MBASE_XA_END_1,
-  SPD_MID_DB_MBASE_XA_PREPARE_1,
-  SPD_MID_DB_MBASE_XA_ROLLBACK_1,
-  SPD_MID_DB_MYSQL_UTIL_APPEND_COLUMN_VALUE_1,
-  SPD_MID_DB_MYSQL_UTIL_APPEND_COLUMN_VALUE_2,
-  SPD_MID_DB_OPEN_ITEM_INT_1,
-  SPD_MID_DB_OPEN_ITEM_STRING_1,
-  SPD_MID_DB_ORACLE_EXEC_QUERY_1,
-  SPD_MID_DB_ORACLE_GET_ERROR_1,
-  SPD_MID_DB_ORACLE_INIT_1,
-  SPD_MID_DB_ORACLE_INIT_2,
-  SPD_MID_DB_ORACLE_RESULT_FETCH_ROW_FROM_TMP_TABLE_1,
-  SPD_MID_DB_ORACLE_RESULT_FETCH_ROW_FROM_TMP_TABLE_2,
-  SPD_MID_DB_ORACLE_RESULT_FETCH_ROW_FROM_TMP_TABLE_3,
-  SPD_MID_DB_ORACLE_ROW_APPEND_ESCAPED_TO_STR_1,
-  SPD_MID_DB_ORACLE_ROW_INIT_1,
-  SPD_MID_DB_ORACLE_ROW_INIT_2,
-  SPD_MID_DB_ORACLE_UTIL_APPEND_COLUMN_VALUE_1,
-  SPD_MID_DB_ORACLE_UTIL_APPEND_COLUMN_VALUE_2,
-  SPD_MID_DB_ORACLE_UTIL_OPEN_ITEM_FUNC_1,
-  SPD_MID_DB_ORACLE_UTIL_OPEN_ITEM_FUNC_2,
-  SPD_MID_DB_ORACLE_UTIL_OPEN_ITEM_FUNC_3,
-  SPD_MID_DB_QUERY_1,
-  SPD_MID_DB_STORE_RESULT_1,
-  SPD_MID_DB_STORE_RESULT_2,
-  SPD_MID_DB_STORE_RESULT_3,
-  SPD_MID_DB_STORE_RESULT_4,
-  SPD_MID_DB_STORE_RESULT_5,
-  SPD_MID_DB_STORE_RESULT_FOR_REUSE_CURSOR_1,
-  SPD_MID_DB_UDF_COPY_TABLES_1,
-  SPD_MID_DB_UDF_PING_TABLE_1,
-  SPD_MID_DB_UDF_PING_TABLE_2,
-  SPD_MID_DB_UDF_PING_TABLE_APPEND_MON_NEXT_1,
-  SPD_MID_DB_UDF_PING_TABLE_APPEND_MON_NEXT_2,
-  SPD_MID_DB_UDF_PING_TABLE_MON_NEXT_1,
-  SPD_MID_DIRECT_SQL_BODY_1,
-  SPD_MID_DIRECT_SQL_INIT_BODY_1,
-  SPD_MID_DISCOVER_TABLE_STRUCTURE_1,
-  SPD_MID_FIELDS_CREATE_CONN_HOLDER_1,
-  SPD_MID_FIELDS_CREATE_LINK_IDX_CHAIN_1,
-  SPD_MID_FIELDS_CREATE_LINK_IDX_HOLDER_1,
-  SPD_MID_GET_INIT_ERROR_TABLE_1,
-  SPD_MID_GET_LGTM_TBLHND_SHARE_1,
-  SPD_MID_GET_PING_TABLE_MON_1,
-  SPD_MID_GET_PING_TABLE_TGT_1,
-  SPD_MID_GET_PT_SHARE_1,
-  SPD_MID_GET_PT_SHARE_2,
-  SPD_MID_GET_SHARE_1,
-  SPD_MID_GET_SHARE_2,
-  SPD_MID_GET_TRX_1,
-  SPD_MID_GET_TRX_10,
-  SPD_MID_GET_TRX_2,
-  SPD_MID_GET_TRX_3,
-  SPD_MID_GET_TRX_4,
-  SPD_MID_GET_TRX_5,
-  SPD_MID_GET_TRX_6,
-  SPD_MID_GET_TRX_7,
-  SPD_MID_GET_TRX_8,
-  SPD_MID_GET_TRX_9,
-  SPD_MID_HANDLERSOCKET_HANDLER_INIT_1,
-  SPD_MID_HANDLERSOCKET_HANDLER_INIT_2,
-  SPD_MID_HANDLERSOCKET_HANDLER_SPIDER_HANDLERSOCKET_HANDLER_1,
-  SPD_MID_HANDLERSOCKET_SHARE_CREATE_COLUMN_NAME_STR_1,
-  SPD_MID_HANDLERSOCKET_SHARE_CREATE_TABLE_NAMES_STR_1,
-  SPD_MID_HANDLERSOCKET_SHARE_CREATE_TABLE_NAMES_STR_2,
-  SPD_MID_HANDLERSOCKET_SHARE_CREATE_TABLE_NAMES_STR_3,
-  SPD_MID_HANDLERSOCKET_SHARE_INIT_1,
-  SPD_MID_HANDLERSOCKET_SHARE_SPIDER_HANDLERSOCKET_SHARE_1,
-  SPD_MID_HA_SPIDER_COND_PUSH_1,
-  SPD_MID_HA_SPIDER_CREATE_1,
-  SPD_MID_HA_SPIDER_CREATE_2,
-  SPD_MID_HA_SPIDER_CREATE_BULK_ACCESS_LINK_1,
-  SPD_MID_HA_SPIDER_FT_INIT_EXT_1,
-  SPD_MID_HA_SPIDER_HA_SPIDER_1,
-  SPD_MID_HA_SPIDER_HA_SPIDER_2,
-  SPD_MID_HA_SPIDER_INFO_PUSH_1,
-  SPD_MID_HA_SPIDER_MULTI_RANGE_READ_NEXT_FIRST_1,
-  SPD_MID_HA_SPIDER_MULTI_RANGE_READ_NEXT_FIRST_2,
-  SPD_MID_HA_SPIDER_MULTI_RANGE_READ_NEXT_FIRST_3,
-  SPD_MID_HA_SPIDER_OPEN_1,
-  SPD_MID_HA_SPIDER_OPEN_2,
-  SPD_MID_HA_SPIDER_OPEN_3,
-  SPD_MID_HA_SPIDER_OPEN_4,
-  SPD_MID_HA_SPIDER_OPEN_5,
-  SPD_MID_HA_SPIDER_OPEN_6,
-  SPD_MID_HA_SPIDER_OPEN_7,
-  SPD_MID_INCREASE_LONGLONG_LIST_1,
-  SPD_MID_INCREASE_LONG_LIST_1,
-  SPD_MID_INCREASE_NULL_STRING_LIST_1,
-  SPD_MID_INCREASE_STRING_LIST_1,
-  SPD_MID_MBASE_COPY_TABLE_INIT_1,
-  SPD_MID_MBASE_HANDLER_APPEND_KEY_COLUMN_TYPES_1,
-  SPD_MID_MBASE_HANDLER_APPEND_MATCH_AGAINST_1,
-  SPD_MID_MBASE_HANDLER_APPEND_TMP_TABLE_AND_SQL_FOR_BKA_1,
-  SPD_MID_MBASE_HANDLER_APPEND_UNION_TABLE_AND_SQL_FOR_BKA_1,
-  SPD_MID_MBASE_HANDLER_INIT_1,
-  SPD_MID_MBASE_HANDLER_INIT_2,
-  SPD_MID_MBASE_HANDLER_INIT_3,
-  SPD_MID_MBASE_HANDLER_INIT_4,
-  SPD_MID_MBASE_HANDLER_INIT_5,
-  SPD_MID_MBASE_HANDLER_INIT_6,
-  SPD_MID_MBASE_HANDLER_INIT_7,
-  SPD_MID_MBASE_HANDLER_INIT_8,
-  SPD_MID_MBASE_HANDLER_INIT_9,
-  SPD_MID_MBASE_HANDLER_INIT_UNION_TABLE_NAME_POS_1,
-  SPD_MID_MBASE_HANDLER_SET_SQL_FOR_EXEC_1,
-  SPD_MID_MBASE_HANDLER_SET_UNION_TABLE_NAME_POS_1,
-  SPD_MID_MBASE_HANDLER_SPIDER_MBASE_HANDLER_1,
-  SPD_MID_MBASE_SHARE_APPEND_SHOW_INDEX_1,
-  SPD_MID_MBASE_SHARE_APPEND_SHOW_INDEX_2,
-  SPD_MID_MBASE_SHARE_APPEND_SHOW_RECORDS_1,
-  SPD_MID_MBASE_SHARE_APPEND_SHOW_TABLE_STATUS_1,
-  SPD_MID_MBASE_SHARE_APPEND_SHOW_TABLE_STATUS_2,
-  SPD_MID_MBASE_SHARE_CREATE_COLUMN_NAME_STR_1,
-  SPD_MID_MBASE_SHARE_CREATE_TABLE_NAMES_STR_1,
-  SPD_MID_MBASE_SHARE_CREATE_TABLE_NAMES_STR_2,
-  SPD_MID_MBASE_SHARE_CREATE_TABLE_NAMES_STR_3,
-  SPD_MID_MBASE_SHARE_DISCOVER_TABLE_STRUCTURE_1,
-  SPD_MID_MBASE_SHARE_INIT_1,
-  SPD_MID_MBASE_SHARE_INIT_2,
-  SPD_MID_MBASE_SHARE_INIT_3,
-  SPD_MID_MBASE_SHARE_INIT_4,
-  SPD_MID_MBASE_SHARE_SPIDER_MBASE_SHARE_1,
-  SPD_MID_OPEN_ALL_TABLES_1,
-  SPD_MID_OPEN_SYS_TABLE_1,
-  SPD_MID_ORACLE_COPY_TABLE_COPY_ROWS_1,
-  SPD_MID_ORACLE_COPY_TABLE_COPY_ROWS_2,
-  SPD_MID_ORACLE_COPY_TABLE_COPY_ROWS_3,
-  SPD_MID_ORACLE_COPY_TABLE_COPY_ROWS_4,
-  SPD_MID_ORACLE_COPY_TABLE_INIT_1,
-  SPD_MID_ORACLE_COPY_TABLE_INIT_2,
-  SPD_MID_ORACLE_HANDLER_APPEND_KEY_COLUMN_TYPES_1,
-  SPD_MID_ORACLE_HANDLER_APPEND_MATCH_AGAINST_1,
-  SPD_MID_ORACLE_HANDLER_APPEND_TMP_TABLE_AND_SQL_FOR_BKA_1,
-  SPD_MID_ORACLE_HANDLER_APPEND_UNION_TABLE_AND_SQL_FOR_BKA_1,
-  SPD_MID_ORACLE_HANDLER_INIT_1,
-  SPD_MID_ORACLE_HANDLER_INIT_2,
-  SPD_MID_ORACLE_HANDLER_INIT_3,
-  SPD_MID_ORACLE_HANDLER_INIT_4,
-  SPD_MID_ORACLE_HANDLER_INIT_5,
-  SPD_MID_ORACLE_HANDLER_INIT_6,
-  SPD_MID_ORACLE_HANDLER_INIT_7,
-  SPD_MID_ORACLE_HANDLER_INIT_8,
-  SPD_MID_ORACLE_HANDLER_INIT_9,
-  SPD_MID_ORACLE_HANDLER_INIT_UNION_TABLE_NAME_POS_1,
-  SPD_MID_ORACLE_HANDLER_SET_SQL_FOR_EXEC_1,
-  SPD_MID_ORACLE_HANDLER_SET_UNION_TABLE_NAME_POS_1,
-  SPD_MID_ORACLE_HANDLER_SPIDER_ORACLE_HANDLER_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_AUTOINC_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_INDEX_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_INDEX_2,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_LAST_INSERT_ID_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_LAST_INSERT_ID_2,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_RECORDS_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_TABLE_STATUS_1,
-  SPD_MID_ORACLE_SHARE_APPEND_SHOW_TABLE_STATUS_2,
-  SPD_MID_ORACLE_SHARE_CREATE_COLUMN_NAME_STR_1,
-  SPD_MID_ORACLE_SHARE_CREATE_TABLE_NAMES_STR_1,
-  SPD_MID_ORACLE_SHARE_CREATE_TABLE_NAMES_STR_2,
-  SPD_MID_ORACLE_SHARE_CREATE_TABLE_NAMES_STR_3,
-  SPD_MID_ORACLE_SHARE_INIT_1,
-  SPD_MID_ORACLE_SHARE_INIT_2,
-  SPD_MID_ORACLE_SHARE_INIT_3,
-  SPD_MID_ORACLE_SHARE_INIT_4,
-  SPD_MID_ORACLE_SHARE_SPIDER_ORACLE_SHARE_1,
-  SPD_MID_PARSE_CONNECT_INFO_1,
-  SPD_MID_PING_TABLE_BODY_1,
-  SPD_MID_PING_TABLE_BODY_2,
-  SPD_MID_PING_TABLE_INIT_BODY_1,
-  SPD_MID_PING_TABLE_MON_FROM_TABLE_1,
-  SPD_MID_RELEASE_PING_TABLE_MON_LIST_1,
-  SPD_MID_TRX_ANOTHER_LOCK_TABLES_1,
-  SPD_MID_UDF_COPY_TABLES_CREATE_TABLE_LIST_1,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_1,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_2,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_3,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_4,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_KEY_1,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_1,
-  SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_2,
-  SPD_MID_UDF_GET_COPY_TGT_TABLES_1
-};
-
 #define SPIDER_BACKUP_DASTATUS \
   bool da_status; if (thd) da_status = thd->is_error(); else da_status = FALSE;
 #define SPIDER_RESTORE_DASTATUS \
@@ -600,7 +330,6 @@ typedef struct st_spider_conn_loop_check SPIDER_CONN_LOOP_CHECK;
 /* database connection */
 typedef struct st_spider_conn
 {
-  uint               conn_kind;
   char               *conn_key;
   uint               conn_key_length;
   my_hash_value_type conn_key_hash_value;
@@ -791,6 +520,7 @@ typedef struct st_spider_conn
   SPIDER_CONN_LOOP_CHECK *loop_check_ignored_first;
   SPIDER_CONN_LOOP_CHECK *loop_check_ignored_last;
   SPIDER_CONN_LOOP_CHECK *loop_check_meraged_first;
+  SPIDER_CONN_LOOP_CHECK *loop_check_meraged_last;
 } SPIDER_CONN;
 
 typedef struct st_spider_lgtm_tblhnd_share
@@ -982,22 +712,30 @@ typedef struct st_spider_share
   char               *table_name;
   uint               table_name_length;
   uint               use_count;
+  /**
+    Probably equals `active_link_count`. See also commit ddff602 of
+    https://github.com/nayuta-yanagisawa/spider-history
+
+    FIXME: consider removing it and using `active_link_count` instead.
+  */
   uint               link_count;
+  /* Number of all links, i.e. all remote servers for the spider
+  table. */
   uint               all_link_count;
   uint               link_bitmap_size;
   pthread_mutex_t    mutex;
   pthread_mutex_t    sts_mutex;
   pthread_mutex_t    crd_mutex;
-/*
-  pthread_mutex_t    auto_increment_mutex;
-*/
   TABLE_SHARE        *table_share;
   SPIDER_LGTM_TBLHND_SHARE *lgtm_tblhnd_share;
   my_hash_value_type table_name_hash_value;
   my_hash_value_type table_path_hash_value;
 
+  /* Whether the share has been initialised */
   volatile bool      init;
+  /* Whether an error occurred in initialisation of this share */
   volatile bool      init_error;
+  /* The time of the initialisation error */
   volatile time_t    init_error_time;
   volatile bool      link_status_init;
   uchar              *table_mon_mutex_bitmap;
@@ -1053,10 +791,6 @@ typedef struct st_spider_share
 
   MEM_ROOT           mem_root;
 
-/*
-  volatile bool      auto_increment_init;
-  volatile ulonglong auto_increment_lclval;
-*/
   ha_statistics      stat;
 
   longlong           static_records_for_status;
@@ -1071,17 +805,27 @@ typedef struct st_spider_share
   longlong           additional_table_flags;
   bool               have_recovery_link;
 
+  /** See `mysql_sysvar_sts_bg_mode` */
   int                sts_bg_mode;
+  /** See `mysql_sysvar_sts_interval` */
   double             sts_interval;
+  /** See `mysql_sysvar_sts_mode` */
   int                sts_mode;
+  /** See `mysql_sysvar_sts_sync` */
   int                sts_sync;
   int                store_last_sts;
+  /** See `mysql_sysvar_load_sts_at_startup` */
   int                load_sts_at_startup;
+  /** See `mysql_sysvar_crd_bg_mode` */
   int                crd_bg_mode;
+  /** See `mysql_sysvar_crd_interval` */
   double             crd_interval;
+  /** See `mysql_sysvar_crd_mode` */
   int                crd_mode;
+  /** See `mysql_sysvar_crd_sync` */
   int                crd_sync;
   int                store_last_crd;
+  /** See `mysql_sysvar_load_crd_at_startup` */
   int                load_crd_at_startup;
   int                crd_type;
   double             crd_weight;
@@ -1119,6 +863,7 @@ typedef struct st_spider_share
   longlong           bgs_second_read;
   longlong           first_read;
   longlong           second_read;
+  /** See `mysql_sysvar_auto_increment_mode` */
   int                auto_increment_mode;
   int                use_table_charset;
   int                use_pushdown_udf;
@@ -1129,6 +874,8 @@ typedef struct st_spider_share
   int                read_only_mode;
   int                error_read_mode;
   int                error_write_mode;
+  /* Number of active remote servers, for use in load balancing read
+  connections */
   int                active_link_count;
 #ifdef HA_CAN_FORCE_BULK_UPDATE
   int                force_bulk_update;
@@ -1151,6 +898,8 @@ typedef struct st_spider_share
   char               **tgt_usernames;
   char               **tgt_passwords;
   char               **tgt_sockets;
+  /** The wrapper of target servers, each element has the same
+  possible values as `SPIDER_DBTON::wrapper` */
   char               **tgt_wrappers;
   char               **tgt_ssl_cas;
   char               **tgt_ssl_capaths;
@@ -1168,6 +917,7 @@ typedef struct st_spider_share
   char               **conn_keys;
   long               *tgt_ports;
   long               *tgt_ssl_vscs;
+  /* See SPIDER_LINK_STATUS_* in spd_include.h */
   long               *link_statuses;
   long               *monitoring_bg_flag;
   long               *monitoring_bg_kind;
@@ -1180,6 +930,7 @@ typedef struct st_spider_share
   long               *connect_timeouts;
   long               *net_read_timeouts;
   long               *net_write_timeouts;
+  /* Connection load balancing integer weight */
   long               *access_balances;
   long               *bka_table_name_types;
   long               *strict_group_bys;
@@ -1206,7 +957,6 @@ typedef struct st_spider_share
   uint               *tgt_pk_names_lengths;
   uint               *tgt_sequence_names_lengths;
   uint               *conn_keys_lengths;
-  /* The index in `spider_dbton' of each data node link. */
   uint               *sql_dbton_ids;
 
   uint               server_names_charlen;
@@ -1272,20 +1022,20 @@ typedef struct st_spider_share
   uint               bka_table_name_types_length;
   uint               strict_group_bys_length;
 
-  /* for dbton */
+  /*
+    For dbton. A `SPIDER_SHARE` uses all `SPIDER_DBTON`s with the same
+    wrappers as any its `tgt_wrappers`
+  */
+  /* Specifies which dbtons of the `spider_dbton` to use */
   uchar              dbton_bitmap[spider_bitmap_size(SPIDER_DBTON_SIZE)];
   spider_db_share    *dbton_share[SPIDER_DBTON_SIZE];
+  /* Number of `SPIDER_DBTON`s used */
   uint               use_dbton_count;
-  /* Actual size is `use_dbton_count'. Values are the indices of item
-  in `spider_dbton'. */
+  /* Index of each `SPIDER_DBTON` in `spider_dbton` to use */
   uint               use_dbton_ids[SPIDER_DBTON_SIZE];
-  /* Inverse map of `use_dbton_ids'. */
   uint               dbton_id_to_seq[SPIDER_DBTON_SIZE];
   uint               use_sql_dbton_count;
-  /* Actual size is `use_sql_dbton_count'. Values are the indices of
-  item in `spider_dbton'. */
   uint               use_sql_dbton_ids[SPIDER_DBTON_SIZE];
-  /* Inverse map of `use_sql_dbton_ids'. */
   uint               sql_dbton_id_to_seq[SPIDER_DBTON_SIZE];
 
   SPIDER_ALTER_TABLE alter_table;
@@ -1298,14 +1048,23 @@ typedef struct st_spider_link_pack
   int                        link_idx;
 } SPIDER_LINK_PACK;
 
+/** A struct storing the initialisation error of a table. All
+instances are in `spider_init_error_tables` */
 typedef struct st_spider_init_error_table
 {
+  /* The associated table name */
   char               *table_name;
+  /* Length of the associated table name */
   uint               table_name_length;
+  /* Hash value of the associated table name for lookup */
   my_hash_value_type table_name_hash_value;
+  /* Whether the error has a message */
   bool               init_error_with_message;
+  /* The error message */
   char               init_error_msg[MYSQL_ERRMSG_SIZE];
+  /* The error code */
   volatile int       init_error;
+  /* The error time */
   volatile time_t    init_error_time;
 } SPIDER_INIT_ERROR_TABLE;
 
