@@ -219,7 +219,7 @@ int spider_release_ping_table_mon_list(
   }
   spider_string conv_name_str(buf, conv_name_length + link_idx_str_length + 1,
     system_charset_info);
-  conv_name_str.init_calc_mem(SPD_MID_RELEASE_PING_TABLE_MON_LIST_1);
+  conv_name_str.init_calc_mem(134);
   conv_name_str.length(0);
   conv_name_str.q_append(conv_name, conv_name_length);
   conv_name_str.q_append(link_idx_str, link_idx_str_length);
@@ -267,7 +267,7 @@ int spider_get_ping_table_mon(
     !(table_link_mon = spider_open_sys_table(
       thd, SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
       SPIDER_SYS_LINK_MON_TABLE_NAME_LEN, FALSE, &open_tables_backup,
-      need_lock, &error_num))
+      &error_num))
   ) {
     my_error(error_num, MYF(0));
     goto error;
@@ -325,7 +325,7 @@ create_table_mon:
 
   do {
     if (!(table_mon = (SPIDER_TABLE_MON *)
-      spider_bulk_malloc(spider_current_trx, SPD_MID_GET_PING_TABLE_MON_1, MYF(MY_WME | MY_ZEROFILL),
+      spider_bulk_malloc(spider_current_trx, 35, MYF(MY_WME | MY_ZEROFILL),
         &table_mon, (uint) (sizeof(SPIDER_TABLE_MON)),
         &tmp_share, (uint) (sizeof(SPIDER_SHARE)),
         &tmp_connect_info,
@@ -386,8 +386,7 @@ create_table_mon:
     error_num = spider_sys_index_next_same(table_link_mon, table_key);
   } while (error_num == 0);
   spider_sys_index_end(table_link_mon);
-  spider_close_sys_table(thd, table_link_mon,
-    &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   table_link_mon = NULL;
   table_mon_list->list_size = list_size;
 
@@ -403,8 +402,7 @@ create_table_mon:
 
 error:
   if (table_link_mon)
-    spider_close_sys_table(thd, table_link_mon,
-      &open_tables_backup, need_lock);
+    spider_sys_close_table(thd, &open_tables_backup);
   table_mon = table_mon_list->first;
   table_mon_list->first = NULL;
   table_mon_list->current = NULL;
@@ -446,7 +444,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
 
   SPD_INIT_ALLOC_ROOT(&mem_root, 4096, 0, MYF(MY_WME));
   if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *)
-    spider_bulk_malloc(spider_current_trx, SPD_MID_GET_PING_TABLE_TGT_1, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, 36, MYF(MY_WME | MY_ZEROFILL),
       &table_mon_list, (uint) (sizeof(SPIDER_TABLE_MON_LIST)),
       &tmp_share, (uint) (sizeof(SPIDER_SHARE)),
       &tmp_connect_info,
@@ -474,7 +472,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   if (
     !(table_tables = spider_open_sys_table(
       thd, SPIDER_SYS_TABLES_TABLE_NAME_STR,
-      SPIDER_SYS_TABLES_TABLE_NAME_LEN, FALSE, &open_tables_backup, need_lock,
+      SPIDER_SYS_TABLES_TABLE_NAME_LEN, FALSE, &open_tables_backup,
       error_num))
   ) {
     my_error(*error_num, MYF(0));
@@ -507,13 +505,12 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
     (*error_num = spider_get_sys_tables_connect_info(
       table_tables, tmp_share, 0, &mem_root)) ||
     (*error_num = spider_get_sys_tables_link_status(
-      table_tables, tmp_share, 0, &mem_root))
+      table_tables, tmp_share->link_statuses, &mem_root))
   ) {
     table_tables->file->print_error(*error_num, MYF(0));
     goto error;
   }
-  spider_close_sys_table(thd, table_tables,
-    &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   table_tables = NULL;
 
   if (
@@ -576,8 +573,7 @@ error_receptor_mutex_init:
 error_caller_mutex_init:
 error:
   if (table_tables)
-    spider_close_sys_table(thd, table_tables,
-      &open_tables_backup, need_lock);
+    spider_sys_close_table(thd, &open_tables_backup);
   free_root(&mem_root, MYF(0));
   if (table_mon_list)
   {
@@ -594,11 +590,9 @@ SPIDER_CONN *spider_get_ping_table_tgt_conn(
 ) {
   SPIDER_CONN *conn;
   DBUG_ENTER("spider_get_ping_table_tgt_conn");
-  if (
-    !(conn = spider_get_conn(
-      share, 0, share->conn_keys[0], trx, NULL, FALSE, FALSE,
-      SPIDER_CONN_KIND_MYSQL, error_num))
-  ) {
+  if (!(conn= spider_get_conn(share, 0, share->conn_keys[0], trx, NULL, FALSE,
+                              FALSE, error_num)))
+  {
     my_error(ER_CONNECT_TO_FOREIGN_DATA_SOURCE, MYF(0),
       share->server_names[0]);
     *error_num = ER_CONNECT_TO_FOREIGN_DATA_SOURCE;
@@ -793,11 +787,10 @@ int spider_get_ping_table_gtid_pos(
     goto error_sys_index_end;
   }
 #ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
-  spider_close_sys_table(thd, table_gtid_pos,
+  spider_sys_close_table(thd, table_gtid_pos,
     &open_tables_backup_gtid_pos, need_lock);
 #endif
-  spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
-    need_lock);
+  spider_sys_close_table(thd, &open_tables_backup_tables);
 
   DBUG_RETURN(0);
 
@@ -808,13 +801,12 @@ error_get_sys_tables_link_status:
 error_sys_index_end:
 error_get_sys_table_by_idx:
 #ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
-  spider_close_sys_table(thd, table_gtid_pos,
+  spider_sys_close_table(thd, table_gtid_pos,
     &open_tables_backup_gtid_pos,
     need_lock);
 error_open_table_gtid_pos:
 #endif
-  spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
-    need_lock);
+  spider_sys_close_table(thd, &open_tables_backup_tables);
 error_open_table_tables:
   DBUG_RETURN(error_num);
 }
@@ -835,7 +827,7 @@ int spider_init_ping_table_mon_cache(
     !(table_link_mon = spider_open_sys_table(
       thd, SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
       SPIDER_SYS_LINK_MON_TABLE_NAME_LEN, FALSE, &open_tables_backup,
-      need_lock, &error_num))
+      &error_num))
   ) {
     my_error(error_num, MYF(0));
     goto error_open_sys_table;
@@ -923,7 +915,7 @@ int spider_init_ping_table_mon_cache(
     spider_mon_table_cache_version = spider_mon_table_cache_version_req;
   }
   pthread_mutex_unlock(&spider_mon_table_cache_mutex);
-  spider_close_sys_table(thd, table_link_mon, &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   DBUG_RETURN(0);
 
 error_push_dynamic:
@@ -932,7 +924,7 @@ error_sys_index_next:
   spider_sys_index_end(table_link_mon);
 error_sys_index_first:
   pthread_mutex_unlock(&spider_mon_table_cache_mutex);
-  spider_close_sys_table(thd, table_link_mon, &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
 error_open_sys_table:
   DBUG_RETURN(error_num);
 }
@@ -1024,8 +1016,8 @@ long long spider_ping_table_body(
   int static_link_id_length = 0;
   bool get_lock = FALSE, status_changed_to_ng = FALSE;
   DBUG_ENTER("spider_ping_table_body");
-  conv_name.init_calc_mem(SPD_MID_PING_TABLE_BODY_1);
-  tmp_str.init_calc_mem(SPD_MID_PING_TABLE_BODY_2);
+  conv_name.init_calc_mem(135);
+  tmp_str.init_calc_mem(247);
   conv_name.length(0);
   server_id = global_system_variables.server_id;
   if (
@@ -1225,9 +1217,9 @@ long long spider_ping_table_body(
               conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
             spider_sys_update_tables_link_status(trx->thd,
               conv_name.c_ptr(), conv_name_length, link_idx,
-              SPIDER_LINK_STATUS_NG, TRUE);
+              SPIDER_LINK_STATUS_NG);
             spider_sys_log_tables_link_failed(trx->thd,
-              conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+              conv_name.c_ptr(), conv_name_length, link_idx);
             status_changed_to_ng = TRUE;
           }
 /*
@@ -1298,9 +1290,9 @@ long long spider_ping_table_body(
                 conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
               spider_sys_update_tables_link_status(trx->thd,
                 conv_name.c_ptr(), conv_name_length, link_idx,
-                SPIDER_LINK_STATUS_NG, TRUE);
+                SPIDER_LINK_STATUS_NG);
               spider_sys_log_tables_link_failed(trx->thd,
-                conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+                conv_name.c_ptr(), conv_name_length, link_idx);
               status_changed_to_ng = TRUE;
             }
 /*
@@ -1361,9 +1353,9 @@ long long spider_ping_table_body(
                 conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
               spider_sys_update_tables_link_status(trx->thd,
                 conv_name.c_ptr(), conv_name_length, link_idx,
-                SPIDER_LINK_STATUS_NG, TRUE);
+                SPIDER_LINK_STATUS_NG);
               spider_sys_log_tables_link_failed(trx->thd,
-                conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+                conv_name.c_ptr(), conv_name_length, link_idx);
               status_changed_to_ng = TRUE;
             }
 /*
@@ -1463,7 +1455,7 @@ my_bool spider_ping_table_init_body(
   }
 
   if (!(mon_table_result = (SPIDER_MON_TABLE_RESULT *)
-    spider_malloc(spider_current_trx, SPD_MID_PING_TABLE_INIT_BODY_1, sizeof(SPIDER_MON_TABLE_RESULT),
+    spider_malloc(spider_current_trx, 11, sizeof(SPIDER_MON_TABLE_RESULT),
       MYF(MY_WME | MY_ZEROFILL)))
   ) {
     strcpy(message, "spider_ping_table() out of memory");
@@ -1606,7 +1598,7 @@ int spider_ping_table_mon_from_table(
   buf[conv_name_length + link_idx_str_length] = '\0';
   spider_string conv_name_str(buf, conv_name_length + link_idx_str_length + 1,
     system_charset_info);
-  conv_name_str.init_calc_mem(SPD_MID_PING_TABLE_MON_FROM_TABLE_1);
+  conv_name_str.init_calc_mem(136);
   conv_name_str.length(0);
   conv_name_str.q_append(conv_name, conv_name_length);
   conv_name_str.q_append(link_idx_str, link_idx_str_length + 1);
@@ -1717,9 +1709,9 @@ int spider_ping_table_mon_from_table(
                   link_idx));
                 share->link_statuses[link_idx] = SPIDER_LINK_STATUS_NG;
                 spider_sys_update_tables_link_status(thd, conv_name,
-                  conv_name_length, link_idx, SPIDER_LINK_STATUS_NG, need_lock);
+                  conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
                 spider_sys_log_tables_link_failed(thd, conv_name,
-                  conv_name_length, link_idx, need_lock);
+                  conv_name_length, link_idx);
               }
 /*
               pthread_mutex_unlock(&table_mon_list->update_status_mutex);
